#include <detect.h>

/*----------------------------------------------------------------------*
 *      Function: WORD Ck_t_kont(t_kont) -- reduces t_kont             *
 *----------------------------------------------------------------------*
 *      Arguments:      WORD t_kont - number of counts from beginning  *
 *                      or end of record to reference time of event     *
 *----------------------------------------------------------------------*
 *      Returns:        t_kont                                          *
 *----------------------------------------------------------------------*
 *      Fatal Errors:   This routine does not have fatal exits          *
 *----------------------------------------------------------------------*
 *      Ck_t_kont is implemented to process records that are one minute *
 *       or longer in length.  For such records, it reduces t_kont to   *
 *       less than the number of counts per minute, and adjust the time *
 *       field that was read in the record header accordingly.          *
 *----------------------------------------------------------------------*/

WORD Ck_t_kont(t_kont)
WORD t_kont;
{
	LONG min_cor;

	if (con_ptr->sam_sec>0) {

		min_cor = t_kont / (con_ptr->sam_sec * 60);

		if (min_cor == 0)
		    return(t_kont);

		t_kont -= 60L * con_ptr->sam_sec * min_cor;

	} else {

		/* printf("org t_kont=%d ",t_kont); */

		min_cor = (((LONG) t_kont) * (con_ptr->ms_sam / 1000L));
		t_kont = (min_cor % 60L) / (con_ptr->ms_sam / 1000L);
		min_cor = min_cor / 60L; /* Calculate minutes */

		/* printf("fix t_kont=%d min_cor=%d\n",t_kont,min_cor); */

		if (min_cor == 0)
		    return(t_kont);
	}

/*-----------If the correction != 0, header time must be adjusted -----*/

	etime.min += min_cor;

	while (etime.min<0)  {	etime.min+=60;		etime.hr--;	}
	while (etime.min>59) {	etime.min-=60;		etime.hr++;	}
	while (etime.hr<0)   {	etime.hr+=24;		etime.day--;	}
	while (etime.hr>23)  {	etime.hr-=24;		etime.day++;	}

	while (etime.day>etime.day_yr) {
		etime.day -= etime.day_yr-1;
		etime.yr++;
	}

	while (etime.day<1) {
		etime.yr--;
		etime.day += etime.prv_yr;	/* Fix 3-Feb-89/SH */
	}

	return(t_kont);
}

#ifdef JNMCOMMENT
/****************************************************************************

			EXPLANATION OF VARIABLES

	etime.day-- the time (day) that was read from the header of the seismic
		    data record.

	etime.hr-- the time (hour) that was read from the header of the seismic
		    data record.

	etime.min-- the time (min) that was read from the header of the seismic
		    data record.

	etime.prv_yr-- the number of days in the previous year.

	etime.yr -- the time (year) that was read from the header of the
		     seismic data record.

	ms_sam  -- Msecs per sample - used for sample rates less than one

	min_cor -- the correction (minute) that is applied to the time in the 
		    header of the seismic data record.

	prev_yr -- the number of days in the previous year.

	sam_sec -- digitizing rate of the current seismic data record. Samples
		    per second.

	t_kont  -- the number of samples from a reference minute (of the 
		    record header) to the reference point of the signal.
		    See Onset (t_samp).

*****************************************************************************/
#endif
