#include <detect.h>

/*----------------------------------------------------------------------*
 *	Function: BOOL Event() - Detect events				*
 *----------------------------------------------------------------------*
 *	Arguments:	No arguments					*
 *----------------------------------------------------------------------*
 *	Returns:	TRUE - if event declared			*
 *			FALSE - no event declared			*
 *----------------------------------------------------------------------*
 *	Fatal Errors:	This routine does not have fatal exits		*
 *----------------------------------------------------------------------*
 *	Detects signals using thresholds th1 and th2. For n_hits = 4	*
 *	 an event may be detected if 4 values are greater than th2, or	*
 *	 if 3 values are greater than th2, and one (or more) of the 3	*
 *	 is greater than th1 (see O-F report 83-785).			*
 *----------------------------------------------------------------------*/

BOOL Event() {

	WORD j, m;
	WORD tfst_flg;
	LONG ab_amp;

	ab_amp = absval(con_ptr->s_amp);

	if (con_ptr->jj > 3)
	    con_ptr->jj = 0;
					 /* Note 1 */
	con_ptr->abuf_rec[con_ptr->jj] = con_ptr->last_rec;
	con_ptr->abuf_sc[con_ptr->jj] = con_ptr->last_sc;
	con_ptr->abuf_amp[con_ptr->jj] = con_ptr->last_amp;
	con_ptr->abuf_tim[con_ptr->jj++] = con_ptr->last_tim;
 					/* NB jj incremented   */
	con_ptr->last_sc = con_ptr->s_sum_sc;
	con_ptr->last_amp = con_ptr->s_amp;
	con_ptr->last_tim = con_ptr->tim_of_max;
	con_ptr->last_rec = con_ptr->rec_of_max;

 					/* Note 2 */
	if (con_ptr->evon) {	       /* evon set=TRUE in Ibingo, =FALSE in
	    				  Count_dn */
	    if (con_ptr->fst_flg > (con_ptr->lst_pt2 = con_ptr->lst_pt))
	        con_ptr->lst_pt2 += E_B;

	    if ((con_ptr->lst_pt2 - con_ptr->fst_flg) == 7) {
	        con_ptr->sumdsc += con_ptr->s_sum_sc;
	        Wbuff();
	        con_ptr->icheck = TRUE;  /* Inhibit writing to buffers */
	        return(TRUE);	       /* Event Declared! */
	    }

	    if ((con_ptr->lst_pt2 - con_ptr->fst_flg) < 8) {
	        con_ptr->sumdsc += con_ptr->s_sum_sc;
	        Wbuff();
	        return(FALSE);
	    }

					/* Note 3 */
	    if (con_ptr->icheck)
	        return(FALSE);

	    con_ptr->icheck = TRUE;
	    return(TRUE);

	}

/*--------------Was event possible set in previous pass(es)?------------------*/
 					/* Note 4 */

	if (!con_ptr->epf) {

	    if (ab_amp < con_ptr->th2)
	        return(FALSE);
	    if (ab_amp < con_ptr->th1)
	        th_wt = 1;
	    else
	        th_wt = 2;

	    con_ptr->epf = TRUE;

	    con_ptr->fst_flg = 4;
	    con_ptr->lst_flg = 4;
	    con_ptr->lst_pt = 3;

	    m = con_ptr->jj;

	    if (m > 3)
	        m = 0;

/*-----A P-T value is >= th2, hence a candidate signal just began---------*/
/*---- --------------Load large buffers from abbrev buffers----------------*/
/*-------These two different buffer types are a holdover from FORTRAN -----*/

	    con_ptr->sumdsc = -con_ptr->abuf_sc[m];

	    for (j = 0; j < 4; j++) {
	        if (m > 3)
	    	m = 0;

	        con_ptr->sumdsc += con_ptr->abuf_sc[m];
	        con_ptr->buf_rec[j] = con_ptr->abuf_rec[m];
	        con_ptr->buf_sc[j] = con_ptr->sumdsc;
	        con_ptr->buf_amp[j] = con_ptr->abuf_amp[m];
	        con_ptr->buf_tim[j] = con_ptr->abuf_tim[m++];
	    				/* m incremented */
	    }
	    con_ptr->sumdsc += con_ptr->s_sum_sc;

	    Wbuff();
	    return(FALSE);
	}

	con_ptr->sumdsc += con_ptr->s_sum_sc;

/*--------------Event possible flag set; however, is current----------------*/
/*----------------------P-T value >= smallest threshold?---------------------*/

 					/* Note 5 */
	if (ab_amp >= con_ptr->th2) {
	    th_wt = 1;		       /* Assign weight=1 to current P-T value*/

/*------------------------------  >= largest threshold?--------------------*/

	    if (ab_amp >= con_ptr->th1)
	        th_wt = 2;		       /* Weight=2 */

/*----------------Is interval between P-T values too small?-----------------*/
/*----------------If so, assign weight=0 to current P-T value---------------*/

	    if ((con_ptr->sumdsc - con_ptr->buf_sc[con_ptr->lst_flg])
	    	<= con_ptr->filhi) {
	        th_wt = 0;
	        Wbuff();
	        return(FALSE);
	    }

/*----------------Is interval between P-T values too large?-----------------*/
/*----------------If so, move beginning of window to current P-T value------*/

	    if ((con_ptr->sumdsc - con_ptr->buf_sc[con_ptr->lst_flg])
	    	>= con_ptr->fillo) {
	        Wbuff();
	        con_ptr->fst_flg = con_ptr->lst_pt;
	        con_ptr->lst_flg = con_ptr->lst_pt;
	        return(FALSE);
	    }
						/* Note 6 */
/*----------------Is this the second P-T value of the candidate signal, ------*/
/*-------------------with no intervening ones? (a special case) --------------*/
/*----------------If so, write buffer and set lst_flg = lst_pt----------------*/

	    if (con_ptr->fst_flg == con_ptr->lst_flg) {
	        Wbuff();
	        con_ptr->lst_flg = con_ptr->lst_pt;
	        return(FALSE);
	    }

						/* Note 7 */
/*------------------Current P-T value >= smallest threshold,----------------*/
/*-------nevertheless slide window, if necessary, to satisfy time criterion---*/
/*-----------------------------Set lst_flg = lst_pt ------------------------*/

	    while ((con_ptr->sumdsc - con_ptr->buf_sc[con_ptr->fst_flg])
	    	> con_ptr->iwin) {
	        con_ptr->indx = con_ptr->fst_flg;
	        con_ptr->lst_flg2 = con_ptr->lst_flg;
	        if ((con_ptr->indx + 1) > con_ptr->lst_flg)
	    	con_ptr->lst_flg2 = con_ptr->lst_flg + E_B;
	        for (j = (con_ptr->indx + 1); j <= con_ptr->lst_flg2; j++) {
	        	/* Is fst_flg == largest index of buffer ? */
	    	if (con_ptr->fst_flg == B_M1)
	    	    con_ptr->fst_flg = -1;
	        	/* changed from ==, 8 jul 86, jnm */
	    	if (con_ptr->buf_flg[++con_ptr->fst_flg] != 0)
	    	    break;	       /* fst_flg incremented */
	        }
	    }
	    Wbuff();

	    con_ptr->lst_flg = con_ptr->lst_pt;
	    con_ptr->iset = FALSE;
	    con_ptr->sumflg = 0;
	    con_ptr->lst_flg2 = con_ptr->lst_flg;

	    if (con_ptr->lst_flg < con_ptr->fst_flg)
	        con_ptr->lst_flg2 = con_ptr->lst_flg + E_B;

/*---------Check for event by evaluating values in flag buffer--------------*/

	    for (j = con_ptr->fst_flg; j <= con_ptr->lst_flg2; j++) {
	        con_ptr->index2 = j;
	        if (con_ptr->index2 > B_M1)
	    	    con_ptr->index2 -= E_B;
	        if (con_ptr->buf_flg[con_ptr->index2] != 0)
	    	    con_ptr->sumflg++;
	        if (con_ptr->buf_flg[con_ptr->index2] == 2)
	    	    con_ptr->iset = TRUE;	/* P-T >= th1 */
	    }
	    if (con_ptr->sumflg < 3)
	        return(FALSE);
	    if (con_ptr->iset) {
	        Ibingo();		       	/* Event detected */
	        return(FALSE);	       		/* But not yet declared */
	    }
	    if (con_ptr->sumflg >= con_ptr->n_hits)
	        Ibingo();
	    return(FALSE);		   /* Event detected but not declared */
	}

/*------Event possible flag is set, however current P-T value is less-------*/
/*------than smallest threshold.  Do not set lst_flg = lst_pt---------------*/

	th_wt = 0;
	Wbuff();

 					/* Note 8 */
/*--------------------Slide window if necessary so that---------------------*/
/*----------first value in window is a flagged value, if possible.----------*/
/*-----------------Otherwise, reset event possible flag.--------------------*/

	while ((con_ptr->buf_sc[con_ptr->lst_pt] -
	    	con_ptr->buf_sc[con_ptr->fst_flg]) > con_ptr->iwin) {
	    con_ptr->lst_flg2 = con_ptr->lst_flg;
	    if (con_ptr->fst_flg > con_ptr->lst_flg)
	        con_ptr->lst_flg2 = con_ptr->lst_flg + E_B;
	    if (con_ptr->fst_flg >= con_ptr->lst_flg2) {

/*------------fst_flg has been rotated to, or (fail safe) past lst_flg, ------*/
/*--------------------and lst_flg is outside of the window.-------------------*/
/*-------------Therefore, there are no flagged P-T values in the window---*/
/*--------------------ie, a candidate signal is not in progress ----------*/

	        con_ptr->epf = FALSE;
	        return(FALSE);
	    }

/*--------------------Rotate fst_flg thru the circular buffer---------------*/
	    for (j = (con_ptr->fst_flg + 1); j <= con_ptr->lst_flg2; j++) {
	        tfst_flg = j;
	        if (tfst_flg > B_M1)
	    	tfst_flg -= E_B;
	        if (con_ptr->buf_flg[tfst_flg] > 0)
	    	break;
	    }
	    con_ptr->fst_flg = tfst_flg;
	}

/*-------------epf remains TRUE, ie, a candidate signal may be in progress--*/
/*-------------However, the candidate signal has not yet been verified--*/

	return(FALSE);
}

#ifdef JNMCOMMENT
/*************************************************************************
				NOTES
	[1]	The abbreviated buffers (abuf) save the last 4 P-T values
		 and their associated time fields.  Four P-T values are 
		 needed because, if the current P-T value is the first 
		 P-T value of a signal that is >= th2, the detector
		 looks back 2 P-T values to test for the onset.  If the
		 earliest of these two P-T values is deemed the onset, then
		 the detector needs the two P-T values before this onset
		 to compute the signal-to-noise series, hence the 4 values.

	[2]	The event on (evon) flag is set=TRUE when routine Ibingo
		 is called.  After evon=TRUE, we want to ensure that
		 enough P-T values have been processed to permit an es-
		 timate of the period of the signal. When enough values
		 have been processed, Event returns TRUE.  It is important
		 to note that Event returns TRUE only once in the interval
		 of the event, unless a new detection in the coda is made.
		 Evon set to False in routine Count_dn when the counter
		 itc <= EV_OFF (ie, evon set to false when (NOR_OUT -
		 EV_OFF) x wait_blk samples have been processed). In the
		 China System, EV_OFF = 2, NOR_OUT = 4, and wait_blk =
		 1014 for the BB and 507 for the SP.

	[3]	Again, we want to return event=TRUE only once per detection.
		 The flag icheck was set to FALSE in routine Ibingo.  There-
		 fore, if there are 8 or more P-T values currently in the
		 window, icheck will be set to TRUE and event returns TRUE.
		 On subsequent calls of routine Event, while evon = TRUE,
		 Event returns FALSE, see [2] above for how evon is controlled.

	[4]	This is where processing of data for detection of an event
		 begins.  The event possible flag (epf) is set = TRUE
		 when a P-T value in the window is >= th2.  During the
		 time epf  = TRUE, data of the large buffers are processed
		 and data are written into them by Wbuff.  The concept of 
		 large and abbreviated buffers is a holdover from the 
		 FORTRAN version.

	[5]	How the detector works has been described by Murdock and
		 Hutt in the USGS Open File Report 83-785. In brief, an event
		 will be detected if, in the window, 3 P-T values are >=
		 th2 and one (or more) of them is >= th1; alternatively,
		 an event will be detected if n_hits P-T values are >=
		 th2, where n_hits is > 3.  However, these P-T values
		 must not occur too close together (no closer than filhi
		 samples), or too far apart (no farther apart than fillo
		 samples).

	[6]	This tests a special case and was implemented to enhance speed.
		 The index fst_flg was initialized to lst_flg, and lst_pt was 
		 initialized to fst_flg-1. lst_pt is incremented in wbuff.
		 Therefore, this if statement will return Event = FALSE
		 for the second consecutive P-T value of a candidate
		 signal, assuming the large buffers were loaded from the
		 abbreviated buffers when the first of these two P-T
		 values was processed. Incidently, the concept of abbre-
		 viated buffers and large buffers is a holdover from the
		 FORTRAN version of the code.

	[7]	If the number of samples from the beginning of the window 
		 to the current P-T value is greater than the window length
		 (samples), move the beginning of the window to the next
		 flagged (th_wt > 0) P-T value.  Continue with this process
		 until the difference between the first P-T value in the
		 window and the last P-T value in the window (both of which
		 are flagged) is <= the window length.

	[8]	The current P-T value is less than th2, nevertheless, we
		 may have a signal in progress but not yet detected.  This
		 is certainly true if the distance between the fst_flg and
		 the lst_flg is less than iwin.  Alternatively, if this dist-
		 tance is greater than iwin, we move the end of the window
		 to the (index of the) current P-T value and then check for
		 flagged P-T values in the window.  If there are no flagged
		 values (ie lst_flg is outside of the window) epf is set=FALSE.
		 On the other hand, if there is a flagged P-T value in the
		 window (be it lst_flg or an earlier one), the beginning of
		 the window is moved to this point (ie fst_flg is set to the
		 index of the first flagged P-T value in the window).

*******************************************************************************

			DESCRIPTION OF VARIABLES

	ab_amp	 -- absolute amplitude of the current P-T. Counts.

	abuf_amp -- an abbreviated buffer that contains the signed
		     P-T amplitudes.

	abuf_rec -- an abbreviated buffer that contains (arbitrary) record
		     numbers in which each P-T value occurred.

	abuf_tim -- an abbreviated buffer that contains time information
		     of the P-T values.  This information is represented
		     by the number of sample counts from a reference time.
		     Check for numerical overflow is not made.

	abuf_sc  -- an abbreviated buffer that contains the number of sample 
		     counts between the P-T values.  Here it is important to
		     note that check for numerical overflow is not made.

	B_M1     -- E_B - 1.  The largest index of the large buffers.

	buf_amp  -- a large buffer that contains signed P-T amplitudes.

	buf_flg  -- a large buffer that contains the values of th_wt
		     for each P-T value of the time window. The 
		     parameter th_wt is a weighting value.

	buf_rec  -- a large buffer that contains record numbers in 
		     which the P-T values of the window occurred.

	buf_sc   -- a large buffer that contains the sum of the number of
		     samples.  This clock is initialized when the abbreviated
		     buffers are downloaded into the large buffers.  It
		     is important to note that values of this buffer are
		     not checked for numerical overflow.  For very high
		     sample rates (ie thousands of samples per second),
		     check for impending overflow, with appropriate subsequent
		     action, should be implemented.

	buf_tim   -- a large buffer that contains time information of the P-T 
		      values of the window.  This information is represented by 
		      the number of sample counts from a reference time.  The
		      reference time is obtained from the header of a record.
		      Check for numerical overflow is not made.  In subsequent
		      versions of the detector, one might be able to combine
		      buf_tim and buf_sc.

	E_B       -- size of the large buffers.  The large buffers should
		      be large enough to inhibit wrap-around in the window.
		      For example, if there is a maximum of 10 P-T values
		      per second (Nyquist frequency = 10), and the window
		      is 4 sec long, E_B should be >=   44 (4 P-T values
		      that are loaded from the abbreviated buffers plus 
		      the maximum of 40 of the window).

	epf       -- event possible flag.  It is set=TRUE when a P-T value
		      in the window is >= th2.  Set=FALSE when an event is
		      detected or when no values in the window are >= th2.

	evon	  -- event on flag.  It is set=TRUE in Ibingo. Set=FALSE
		      in Count_dn in the interval of the coda of the event.

	fst_flg	  -- an index to the first flagged P-T value of the window.
		      It is an index to the first P-T value >= th2.

	index2    -- an index that is within the range of the large buffers:
		      If index2 exceeds the size of the buffer, index2 is set=0.
 
	indx      -- saves the value of fst_flg when fst_flg may be changed.

	iset      -- a flag that is set=TRUE when a P-T value of the window
		      >= th1.

	icheck    -- a flag that is set=FALSE in Ibingo, set=TRUE in Event
		      when enough P-T values have been processed (8) to
		      estimate the period of the signal.

	iwin      -- the length of the window (samples).

	jj        -- an index for the abbreviated buffers.
	
	last_amp  -- signed amplitude of the previous (as opposed to 
		      current) P-T.

	last_rec  -- arbitrary record number in which the previous P-T occurred.

	last_sc	  -- the number of samples (time axis) between the two previous
		      P-T values.

	last_tim  -- the sample number of the last P-T value (the sample
		      number is initialized at the beginning of each record).
		      This sample number is used to calculate the time of the
		      P-T value by using the information in the record header.

	lst_flg   -- an index to the last P-T value that is >= th2.

	lst_flg2  -- circular buffers are employed, lst_flg2 is a bound that
		      is corrected for buffer length.

	lst_pt    -- an index to the last P-T value that was processed,
		      it is incremented in routine Wbuff.

	lst_pt2   -- circular buffers are employed, lst_pt2 is a bound that
		      is corrected for buffer length.

	m         -- an index for the abbreviated buffers.

	n_hits    -- the number of P-T values in the window that is required
		      for a detection if they are all >= th2 but < th1.

	rec_of_max-- arbitrary record number in which the current P-T occurred.

	s_amp	  -- signed amplitude of the current P-T. Counts.

	s_sum_sc  -- the number of samples (time axis) between the current 
		      P-T value and the previous one.

	sumdsc	  -- the sum of the s_sum_sc that is initialized when the
		      abbreviated buffers are downloaded. Not checked for 
		      numerical overflow.

	sumflg    -- the number of flagged P-T values (ie, those with an
		      associated th_wt > 0) in the window.

	tfst_flg  -- temporary fst_flg.

	th1       -- the largest threshold for detection of an event.  Typically
		      th1 = 2.0 x twosd, where twosd is an approximation of 
		      twice the sample standard deviation of the P-T values.
		      Counts.
		      
	th2       -- the smallest threshold for detection of an event. Typically
		      th2 = 1.5 x twosd, where twosd is an approximation of 
		      twice the sample standard deviation of the P-T values.
		      Counts.


	th_wt     -- threshold weight.
		      = 2 if current P-T value is >= th1
		      = 1 if current P-T value is >= th2, but < th1
		      = 0 otherwise

	tim_of_max-- the sample number of the current P-T value (the sample
		      number is initialized at the beginning of each record).
		      This sample number is used to calculate the time of the
		      P-T value by using the information in the record header.

*******************************************************************************/
#endif
