#include <detect.h>

/*----------------------------------------------------------------------*
 *	Function: VOID P_two(maxamp) - Estimate average P-T		*
 *----------------------------------------------------------------------*
 *	Arguments:	LONG maxamp - maximum (absolute) of 20		*
 *				successive P-T values < thx		*
 *----------------------------------------------------------------------*
 *	Returns:	Nothing (VOID)					*
 *----------------------------------------------------------------------*
 *	Fatal Errors:	This routine does not have fatal exits		*
 *----------------------------------------------------------------------*
 *	P_two estimates the statistical dispersion of the background	*
 *	 noise and calculates the four thresholds that are used in the	*
 *	 detector.  Remember that P_one finds the maximum of 20 rec-	*
 *	 tified P-T values and normally sends this maximum to P_two.  	*
 *	 P_two averages val_avg (8 or 16) of these maximums. The 	*
 *	 average thus obtained is twosd.  For zero-mean normally 	*
 *	 distributed P-T values, twosd would be an estimate of twice 	*
 *	 the sample standard deviation of the P-T values. (See OF 83-785*
 *	 for a comparison between the measured sample standard devia-	*
 *	 tion and the estimate of it hereby.)  P_two calls xth to cal-	*
 *	 culate the thresholds from twosd.				*
 *----------------------------------------------------------------------*/

VOID P_two(maxamp)
LONG maxamp;
{
	register WORD i,c;
	LONG sum,Xth();

	if(!con_ptr->evon) { /* twosd not calculated while evon=TRUE */

/*-----------------------------Load buffer---------------------------------*/

		con_ptr->tsstak[con_ptr->kk] = maxamp;
		if(++con_ptr->kk == con_ptr->val_avg) con_ptr->kk = 0;

/*-----------------------------Average buffer------------------------------*/

		sum = 0;	/* prepare to average */
#ifndef RAMPUP
		for(i= 0;i < con_ptr->val_avg; i++) sum += con_ptr->tsstak[i];
		c = con_ptr->val_avg;
#else
		c=0;
		for (i=0; i<con_ptr->val_avg; i++) {
			if (con_ptr->tsstak[i]>0) {
				sum += con_ptr->tsstak[i];
				c++;
			}
		}
#endif
		
		switch (c) { /* perform average */
		case 16:
			con_ptr->twosd = sum >> 4;
			break;
		case 8:
			con_ptr->twosd = sum >> 3;
			break;
		default:
/*		we try to avoid 32-bit division			*/
			if (c<=0) con_ptr->twosd=0;
			else	con_ptr->twosd = sum / c;
			break;
		}

				/* Special condition -- if twosd ever
					got to be zero, it would stay there */ 
		if(con_ptr->twosd <= 0) 
			con_ptr->twosd = 1000000L;

/*----------------------------Calculate thresholds-------------------------*/

		con_ptr->th1 = Xth(con_ptr->xth1);
		con_ptr->th2 = Xth(con_ptr->xth2);
		con_ptr->th3 = Xth(con_ptr->xth3);
		con_ptr->thx = Xth(con_ptr->xthx);
	
	}
	return;
}

#ifdef JNMCOMMENT
/****************************************************************************

			DEFINITION OF VARIABLES

	evon	-- event on flag.  Set = TRUE in Ibingo (event detected)
		    Set = FALSE in routine Count_dn when itc is decremented
		    to EV_OFF. EV_OFF typically is 2 (evon set=FALSE
		    when itc - EV_OFF  x wait_blk samples have been
		    processed, where itc typically is 4 and wait_blk
		    typically is 1014).

	kk	-- index for the tsstak.

	maxamp	-- the absolute maximum of 20 successive P-T values that
		    are less than thx.

	sum	-- the sum of the maximum amplitudes that are stored in
		    the buffer tsstak.

	tsstak	-- a buffer that contains the history of the maximum
		    amplitudes that were input by P_one.

	val_avg	-- the number of maximum values (input from P_one) 
		    that is used to estimate each twosd.

******************************************************************************/
#endif
