#include <detect.h> 

/*----------------------------------------------------------------------*
 *	Function: VOID Period(p_kont) - Find "floating point"		*
 *						for output		*
 *----------------------------------------------------------------------*
 *	Arguments:							*	
 *			WORD p_kont - samples in the period		*
 *----------------------------------------------------------------------*
 *	Returns:	Nothing (VOID)					*
 *----------------------------------------------------------------------*
 *	Fatal Errors:	This routine does not have fatal exits		*
 *----------------------------------------------------------------------*
 *	The routine Period calculates the period of the detected signal *
 *	 from the value P_kont.  The purpose of this routine is to 	*
 *	 circumvent floating point operations.				*
 *----------------------------------------------------------------------*/

VOID Period(p_kont)
WORD p_kont;
{
	if (con_ptr->sam_sec>0) {
		p_lval = p_kont/con_ptr->sam_sec;
		p_rval = (p_kont % con_ptr->sam_sec) * con_ptr->ms_sam;
	} else {
		p_lval = p_kont * (con_ptr->ms_sam / 1000);
		p_rval = 0;
		/* printf("p_kont=%d,ms_sam=%d,p_lval=%d\n",
			p_kont,con_ptr->ms_sam,p_lval); */
 	}

	return;
}		

#ifdef JNMCOMMENT
/*************************************************************************

			EXPLANATION OF VARIABLES

	ms_sam	-- milliseconds between successive samples of the digital
		    data.  See Pass_one.
	
	p_kont	-- the average number of samples per cycle in the first four 
		    cycles of the declared signal.  See Onsetq.

	p_lval 	-- the integer part of the period. Seconds.

	p_rval	-- the remainder (decimal fraction) of the period
		    expressed in milliseconds.

	sam_sec -- samples per second of the digital data.
		    See E_detect, Pass_one.

************************************************************************/
#endif
