#include <detect.h>

/*----------------------------------------------------------------------*
 *	Function: WORD Time_f(t_kont,tc) - "floating point"		*
 *						for output		*
 *----------------------------------------------------------------------*
 *	Arguments:							*	
 *			WORD t_kont - onset sample number		*
 *			WORD tc - time correction			*
 *----------------------------------------------------------------------*
 *	Returns:	LONG - onset time				*
 *----------------------------------------------------------------------*
 *	Fatal Errors:	This routine does not have fatal exits		*
 *----------------------------------------------------------------------*
 *	This routine calculates the onset time of the detected signal	*
 *	 from t_kont and tc.  One purpose of this routine is to cir-	*
 *	 cumvent floating point operations.				*
 *----------------------------------------------------------------------*/

LONG Time_f(t_kont,tc)
WORD t_kont;
LONG tc;
{

	LONG reftr,t_lval;

/*-------------------More than one minute per record?-------------------*/
/*		     If so, adjust values read in headers.		*/

	t_kont = Ck_t_kont(t_kont);	

	if (con_ptr->sam_sec>0) {

		t_lval = t_kont/con_ptr->sam_sec;	/* Integer Seconds */
		reftr = t_kont - t_lval * con_ptr->sam_sec; /* Rem in samples */
		t_rval = reftr * con_ptr->ms_sam + etime.msec - tc; /* Rem ms */

	} else {

		t_lval = t_kont * (con_ptr->ms_sam / 1000);
		reftr = t_kont - ((t_lval*1000) / con_ptr->ms_sam);
		t_rval = etime.msec - tc;

		/* printf("time_f t_kont=%d, t_lval=%d, t_rval=%d, reftr=%d\n",
			t_kont,t_lval,t_rval,reftr); */

	}

/*---------------------Obtain remainder < 1000 ms ---------------------*/

	while(t_rval >= 1000) {
		t_lval++;
		t_rval -= 1000; /* milliseconds remainder */
	}

/*---------The time correction could have caused remainder < 0 -------*/

	while(t_rval <= -1000) {
		t_lval--;
		t_rval += 1000; /* milliseconds remainder */
	}

	if(t_rval < 0) {
		t_lval--;
		t_rval += 1000; /* milliseconds remainder */
	}
	
/*--------------Integer seconds relative to header time--------------*/

	t_lval += etime.sec;

	return(t_lval);
}

#ifdef JNMCOMMENT

/***************************************************************************

			EXPLANATION OF VARIABLES

	etime.sec-- the reference time (seconds) of a record header.

	ms_sam --  sample rate expressed as milliseconds per sample.

	sam_sec -- digitizing rate of the current seismic data record.
		    Samples per second.

	reftr	-- remainder of integer division expressed in samples.

	tc	-- the time correction (milliseconds) that is applied to the
		    reference onset.  For BB and SP data of the China System
		    tc = 500 (ms) or 0.

	t_kont	-- the number of samples from a reference time of a record
		    header to the reference onset of a signal.

	t_lval	-- the integer part of the onset time (seconds).

	t_rval	-- the remainder of the onset time expressed in milliseconds.
*****************************************************************************/
#endif
