#include <detect.h>

/*----------------------------------------------------------------------*
 *	Function: LONG Xth(xthi) - Calculate threshold			*
 *		adapted from a routine by R.R. Reynolds 		*
 *----------------------------------------------------------------------*
 *	Arguments:	UWORD xthi - factor				*
 *----------------------------------------------------------------------*
 *	Returns:	Return the threshold value			*
 *----------------------------------------------------------------------*
 *	Fatal Errors:	This routine does not have fatal exits		*
 *----------------------------------------------------------------------*
 *	The routine Xth forms the thresholds from twosd and from the	*
 *	 encoded factors xthi.  Shifts and encoded factors are em-	*
 *	 ployed to circumvent the need for 32 bit multiplication.	*
 *									*
 *	As coded here, the maximum permitted value of xthi is 0377	*
 *----------------------------------------------------------------------*/

LONG Xth(xthi)
register UWORD xthi;  		
{

	LONG th = 0;
	register UWORD x_left;

	x_left = xthi >>3; 	/* remove first three bits */

/*----------Calculate high order part of threshold-------------------*/

	if(x_left) {
		if(x_left & 020) th  = con_ptr->twosd<<4;
		if(x_left & 010) th += con_ptr->twosd<<3;
		if(x_left & 004) th += con_ptr->twosd<<2;
		if(x_left & 002) th += con_ptr->twosd<<1;
		if(x_left & 001) th += con_ptr->twosd;
	}

/*-----------Calculate low order part of threshold------------------*/

	if(xthi & 07) { 	/* use the first 3 bits */
		if(xthi & 01) th += (con_ptr->twosd >>3);
		if(xthi & 02) th += (con_ptr->twosd >>2);
		if(xthi & 04) th += (con_ptr->twosd >>1);
	}
	return(th);
}



#ifdef JNMCOMMENT
/****************************************************************************

			EXPLANATION OF VARIABLES

	twosd	-- an estimate of twice the sample standard deviation of
		    the P-T values (counts).  See P_one, P_two.

	x_left	-- an unsigned 16 bit word that contains xthi, with the
		    first 3 bits of xthi removed.  After shifting to re-
		    move these bits, the remaining that are set in x_left
		    are used to control the amount twosd is multiplied
		    (shifted) to obtain the high order part of the 
		    threshold, th.

	xthi	-- the input encoded factor of twosd.  Octal values.
		    Ocatal values were selected for continuity with the
		    SRO on-line detector.

		   xthi	Value	Factor	   xthi Value	Factor	
		      (octal)  (decimal)      (octal)  (decimal)
			10	1.0		30	3.0
			11	1.125		40	4.0
			12	1.25		50	5.0
			13	1.375		60	6.0
			14	1.5		70	7.0
			15	1.625		100	8.0
			16	1.75		300	24.0
			17	1.875		377	31.875
			20	2.0
*************************************************************************/
#endif
