#include <detect.h>

/*----------------------------------------------------------------------*
 *      Function: VOID main() - main section for detecting data streams *
 *----------------------------------------------------------------------*
 *      Arguments:      Arguments are taken from STDIN                  *
 *----------------------------------------------------------------------*
 *      Returns:        Nothing (VOID)                                  *
 *----------------------------------------------------------------------*
 *      Fatal Errors:   If any mallocs fail, or user types an           *
 *                      interrupt, the program will exit abnormally     *
 *----------------------------------------------------------------------*
 *      This is an example of how to define a main or other subroutine  *
 *      to setup and call the event detector.  This particular example  *
 *      defines an off-line detector which will read through as many    *
 *      different input devices as required, and will do event          *
 *      detection on whatever channels, as desired. It is even possible *
 *      to run multiple detections on the same to channel, to test      *
 *      different parameters.  Processing will continue until there     *
 *      is an end of file on all input devices.                         *
 *                                                                      *
 *      The information used to set up the various event detectors      *
 *      are entered in STDIN as a parameter file, here is an example:   *
 *                                                                      *
 *      /dev/rmtb bb_z 1 0 12 4 2 200 77 17 10 30 5 500 1014 16         *
 *      /dev/rmtb sp_z 2 0 12 8 1 160 20 15 10 15 4 500 0507 8          *
 *                                                                      *
 *      The parameters are in the following order separated by spaces:  *
 *                                                                      *
 *      input_device detector_name rec_type component block_factor      *
 *      filhi fillo iwin xth1 xth2 xth3 xthx n_hits def_tc wait_blk     *
 *      val_avg                                                         *
 *----------------------------------------------------------------------*/

#define MAXDET 12            /* How may event detected channels MAX */
#define DBF_SIZ 2000

struct detstr {
	TEXT detnam[20];         /* Place to store detector name */
	TEXT devnam[30];         /* Device for this detector */
	LONG dev_nbr;           /* Number in device array for this detector */
	LONG recotyp;           /* Record type */
	LONG compon;            /* Component to use */
	LONG block_f;           /* Blocking factor */
	LONG s_filhi, s_fillo, s_iwin, s_xth1, s_xth2, s_xth3, s_xthx,
	s_n_hits, s_def_tc, s_wait_blk, s_val_av;
				/* User's detector parameters */
}               dtectr[MAXDET];	/* One for each detector */

struct detect_info  edete[MAXDET];/* Detector setup data */

struct devlst {
	TEXT lstnam[30];         /* The name of the device */
	FILE *filnum;            /* The file number where it is open()'d */
	LONG d_block;           /* The size of the block */
	LONG b_fact;            /* The blocking factor */
}               dvice[MAXDET];	/* One per device */

UBYTE * in_rec;               /* Our read in buffer */
LONG * mainarray;           /* Main data array */

VOID main() {

	LONG det_nbr, dv_nbr, i, k, maxb;
	LONG det_no, dev_no;
	LONG ikount;
	BOOL any, ev, (*filt)(), FfirSP20(), Fnull();

							/****Note 1****/
/*-----------------Let interupts get us out under UNIX-------------------*/

#ifdef UNIX                  /* Only meaningful on unix */
	extern  onintr();
	if (signal(SIGINT, SIG_IGN) != SIG_IGN)
	    signal(SIGINT, onintr);
#endif
#ifdef __MSDOS__
	int onintr();

	ctrlbrk(onintr);
#endif

/*-----------------Print out current configuration-----------------------*/

	printf("\nCurrent Detector Configuration:\n");

#ifdef RAMPUP
	printf("     RAMPUP=ON ");
#else
	printf("     RAMPUP=OFF ");
#endif

#ifdef PONSET
	printf("PONSET=ON ");
#else
	printf("PONSET=OFF ");
#endif

#ifdef FFILTER
	printf("FFILTER=ON ");
#else
	printf("FFILTER=OFF ");
#endif

#ifdef RESYNCRONIZE
	printf("RESYNCRONIZE=ON ");
#else
	printf("RESYNCRONIZE=OFF ");
#endif

#ifdef ITC_PER_CNTRL
	printf("ITC_PER_CNTRL=ON ");
#else
	printf("ITC_PER_CNTRL=OFF ");
#endif

	printf("\n");

	printf("     EV_OFF=%d NOR_OUT=%d CUR_MAX=%d E_B=%d\n",
		EV_OFF,NOR_OUT,CUR_MAX,E_B);
#ifdef FFILTER
	printf("     MAXSECTION=%d LBSIZE=%d\n",
		MAXSECTION,LBSIZE);
#endif
#ifdef ITC_PER_CNTRL
	printf("     VSP_SPS=%d PER_TRIG=%d ITC_UP=%d\n",
		VSP_SPS,PER_TRIG,ITC_UP);
#endif

	printf("\n");	

/*----------------Initialize variables for loops-------------------------*/

	ikount = 0;                   /* counts number of devices closed, for
					 normal exit */
	maxb = 0;                     /* the maximum blocking factor used
					 herein */
	det_nbr = 0;                  /* number of the detector, incremented
					 in while loop */
	dv_nbr = (-1);                /* number of the device, incremented in
					 while loop */

	mainarray = (LONG *) E_buffer(1000, 40);/* Max lookback 40 */

							/*****Note 2*****/
/*-----------Loop through to get each of our detector parameter lines------*/

	while (!feof(stdin)) {

	    i = scanf("%s %s %ld %ld %ld %ld %ld %ld %lo %lo %lo %lo %ld %ld %ld %ld",
		    dtectr[det_nbr].devnam,
		    dtectr[det_nbr].detnam,
		    &dtectr[det_nbr].recotyp,
		    &dtectr[det_nbr].compon,
		    &dtectr[det_nbr].block_f,
		    &dtectr[det_nbr].s_filhi,
		    &dtectr[det_nbr].s_fillo,
		    &dtectr[det_nbr].s_iwin,
		    &dtectr[det_nbr].s_xth1,
		    &dtectr[det_nbr].s_xth2,
		    &dtectr[det_nbr].s_xth3,
		    &dtectr[det_nbr].s_xthx,
		    &dtectr[det_nbr].s_n_hits,
		    &dtectr[det_nbr].s_def_tc,
		    &dtectr[det_nbr].s_wait_blk,
		    &dtectr[det_nbr].s_val_av);

	    if (i!=16) break;

#ifdef DISP_P
	    printf("Detector #%ld Device %s Channel %s #%ld (rec %ld) ",
		det_nbr,
		    dtectr[det_nbr].devnam, dtectr[det_nbr].detnam,
		dtectr[det_nbr].compon, dtectr[det_nbr].recotyp);
	    printf("Blocking Factor %d\n",
		    dtectr[det_nbr].block_f);
	    printf("filhi=%ld,fillo=%ld,iwin=%ld,xth1=%lo,xth2=%lo,xth3=%lo,\
	       xthx=%lo,n_hits=%ld,\ndef_tc=%ldms,\ wait_blk=%ld,val_avg=%ld\n",
		    dtectr[det_nbr].s_filhi, dtectr[det_nbr].s_fillo,
		    dtectr[det_nbr].s_iwin, dtectr[det_nbr].s_xth1,
		dtectr[det_nbr].s_xth2,
		    dtectr[det_nbr].s_xth3, dtectr[det_nbr].s_xthx,
		    dtectr[det_nbr].s_n_hits, dtectr[det_nbr].s_def_tc,
		    dtectr[det_nbr].s_wait_blk, dtectr[det_nbr].s_val_av);
#endif

							/******Note 3*****/
/*-------------   Calculate maxb for malloc --------------------------------- */

	    if (dtectr[det_nbr].block_f > maxb)
		maxb = dtectr[det_nbr].block_f;

/*----------------More than one detector per device is permitted--------------*/

	    for (i = 0; i <= dv_nbr; i++)
		if (!strcmp(dvice[i].lstnam, dtectr[det_nbr].devnam)) {

/*----------------New detector, same device----------------------------------*/

		    i = (-1);
		    break;
		}

	    if (i != -1) {

							/****Note 4****/
/*---------------New device--------------------------------------------------*/

		strcpy(dvice[++dv_nbr].lstnam, dtectr[det_nbr].devnam);
		dvice[dv_nbr].filnum = NULL;
		if ((dvice[dv_nbr].filnum = fopen(dvice[dv_nbr].lstnam,"rb")
				) != NULL) {
		    printf("Device %s opened\n",
			    dvice[dv_nbr].lstnam);
		}

		else
		    printf("Device %s cannot be opened\n",
			    dvice[dv_nbr].lstnam);

	    }

	    if (dvice[dv_nbr].filnum != NULL) {

							/****Note 5****/
/*----------Put user's parameters into the continuity structure-------------*/

		switch (dtectr[det_nbr].recotyp) {
		    case 1:
			k = 7;
			filt = FfirSP20;
			break;
		    case 2:
			k = 0;
			filt = Fnull;
			break;
		    default:
			k = 0;
			filt = Fnull;
		}

		E_create(&edete[det_nbr], k, dtectr[det_nbr].recotyp,
			dtectr[det_nbr].detnam, mainarray, filt);

		setallparams(&edete[det_nbr],
			dtectr[det_nbr].s_filhi,
			dtectr[det_nbr].s_fillo,
			dtectr[det_nbr].s_iwin,
			dtectr[det_nbr].s_n_hits,
			dtectr[det_nbr].s_xth1,
			dtectr[det_nbr].s_xth2,
			dtectr[det_nbr].s_xth3,
			dtectr[det_nbr].s_xthx,
			dtectr[det_nbr].s_def_tc,
			dtectr[det_nbr].s_wait_blk,
			dtectr[det_nbr].s_val_av);

		Disp_par(&edete[det_nbr]);

		dvice[dv_nbr].d_block = DBF_SIZ;
		dvice[dv_nbr].b_fact = dtectr[det_nbr].block_f;
		dtectr[det_nbr].dev_nbr = dv_nbr;

	    }

	    det_nbr++;

	}

	if (!(in_rec = (UBYTE *) malloc(maxb * DBF_SIZ))) {
	    printf("failure on malloc of rec_str\n");
	    exit(1);
	}

	printf("Detectors created\n");

	any = FALSE;

							/****Note 6****/
/*-----------------------Main loop - process input data--------------------*/
	do {
	    for (dev_no = 0; dev_no <= dv_nbr; dev_no++) {
	    /* check all devices */
		if (dvice[dev_no].filnum >= 0) {
		    any = TRUE;
		    if (k = (fread((void *) in_rec,
				dvice[dev_no].d_block,
				dvice[dev_no].b_fact,
				dvice[dev_no].filnum)) !=
					dvice[dev_no].b_fact) {
			dvice[dev_no].filnum = NULL;
					/* error condition */
			if (k == 0)
			    printf("Drive %s at EOF\n",
				    dvice[dev_no].lstnam);
			close(dvice[dev_no].filnum);
			dtectr[det_nbr].dev_nbr = -1;
			if (ikount++ == dv_nbr)
			    exit(0);
			else
			    continue;      /* try another device */
		    }

							/****Note 7****/
/*------------------------Process and detect each data-----------------------*/

		    for (det_no = 0; det_no < det_nbr; det_no++) {

			if (dtectr[det_no].dev_nbr == dev_no)
			    for (k = 0; k < dvice[dev_no].b_fact; k++) {
				if (E_sroload(&edete[det_no],
					    (UBYTE *) in_rec + (k * DBF_SIZ),
					    dtectr[det_no].compon)) {

				    E_filter(&edete[det_no]);
				    E_detect(&edete[det_no]);
				}
			    }
		    }
		}
	    }
	} while (any == TRUE);        /* True while valid file number exists */
}

#ifdef UNIX
onintr() {                  /* the sole purpose of this routine is to indicate
			       that an interrupt has occurred.  It is called
			       by the UNIX utility signal  in the routine
			       Detmain */
	printf("processing interrupt\n");
	exit(1);
}
#endif
#ifdef __MSDOS__
int onintr(void)
{

	printf("Processing interrupt\n");
	return(0);

}
#endif


#ifdef SHCOMMENT
/******************************************************************************
			Program Notes (see inline code)


	[1]	On UNIX, the program ocassionally had difficulty
		being interrupted and closing up correctly.  This
		signal was inserted to assure that this problem
		would not occur.  It might not be necessary on 
		other versions of unix (not BSD 2.9), or on other
		operating systems.  In any case, an interrupt here forces
		an exit(), which is useful when doing a profile.

	[2]	The event detector information is read from standard in
		with scanf().  No attempts are made to assure data
		accuracy or format.  Please keep your input data
		accurate and always provide the correct number of
		parameters.

	[3]	The maximum buffer size is computed.  This will be the
		size of the largest input block.

	[4]	The devices requested in the input parameters are 
		compared which the current table.  Device names which
		are not identical to the names in the table are 
		declared new devices, and are placed in the tables.
		Do not use alias device names for the same device, and
		be sure that the names are spelled the same, and are
		in the same case, for the same device.

		The device is also opened here, and if it cannot be
		opened, the detectors for this device are automatically
		dropped, and will not be detected.

	[5]	The filtering and lookback information are determined
		baised on the record type, and then the detector structure
		is built via E_create().  The event detector parameters
		are then stowed away in the detector's continuity structure
		with setallparams() (see csmacs.h).

	[6]	All parameters are loaded, and all devices are open.  This
		is the main processing loop.  Each device is read round-robin
		fashion, and one device block is read for each.  This means
		that large blocked devices will process more records than
		smaller ones.  Take this into account when planning the data
		flow through the detector.

	[7]	The above block is deblocked.  All of the event detectors
		which use this device are checked against each block.  If
		the data are pertinent, the data is filtered, and the
		event detector is run.  Event onset information is captured
		via standard out.  If the user wishes more complex onset
		information processing, an onset routine can be pointed to
		by the continuity structure (see Onset()).

******************************************************************************
			Explanation of Variables 

	dtectr - An array of structures whose purpose is to keep track
		of the i/o details of each detector.  These data are
		stored when the configuration/parameters data are read
		in.

	edete - An array of detector structures which will contain all
		of the actual logistical information required by each
		detector.  This is originally set up by e_create, and
		updated and refered to by all of the e_* routines called
		by the event detector.  It contains the continutity 
		structure for each detector, and contains the lookback
		arrays to be used for filtering.  It also contains
		pointers to the data storage arrays, and information
		which is used by the loader/decoder to ensure that
		the detector will match the data which is intended for
		it.

	dvice - An array of structures which contain information for each
		discrete i/o device which is read to get data for the 
		detector.  Housekeeping information for general device
		i/o is kept here.

	det_nbr - This is the current count of detectors in the dtectr and
		edete arrays.  It is incremented and used as an index as
		information is loaded, and used as an index maximum in
		the read loops.

	dv_nbr - This is the current count of devices, and is the index to
		the dvice structure array.  It is used similarly to det_nbr.

	i, k - These are temporary looper and indexer variables.

	maxb - This is used to contain the largest buffer size requested 
		in the input parameters, so that an input buffer of
		sufficient size can be malloc()'d.

	det_no, dev_no - variables corresponding to the detector and device
		arrays, and used as indexes during the read loops.

	ikount - This contains the current count of devices.  This is set
		to the number of devices originally, and is decremented as
		devices reach end of file.  When this is zero, the program
		knows that all data is processed, and it can exit.

	any - A boolean flag.

	filt - A pointer to a function which returns a boolean value.  This 
		is used in the switch statement to assign which filter
		function will be used to filter a particular record type.
		This is passed on into E_create(), to be stored in the
		detector structure for that particular detector.

	mainarray - An array of LONG (longs) is used for storage of the
		decoded data, and is also used for the filtered data.
		This multiple use of buffers is efficient in terms of
		time and memory usage, and is done automatically by the
		decode routines (such as E_cdsnload()), and the filter
		routine E_filter().  If one does not desire filtering,
		be sure to specify a lookback of 0 when the filter is
		setup.  If one wishes lookback to be stored, but will
		not be using the normal lookback facillity, one must 
		still roll the data up to the top of the array once
		the user has filtered it.  This can be done by specifying
		the Fnull() filter to e_create.  Note that if one does
		not call the E_filter() routine, the lookback must be
		retrieved and placed in the storage area by hand by the
		user's program.

	in_rec - This is the buffer which is maxb chars long which is used
		to load the raw unconverted data from the various input
		devices.  The decoder then processes this into useful
		data.
*****************************************************************************/
#endif


