#include <detect.h>

/*----------------------------------------------------------------------*
 *	Function: BOOL FfirBB20(points,lookback,data) - 20SPS BB fir 	*
 *----------------------------------------------------------------------*
 *	Arguments:	WORD points - number of data points in the	*
 *					the current record		*
 *			WORD lookback - amount of lookback required	*
 *					 for this filter		*
 *			LONG *data - array that contains the seismic	*
 *					samples to be processed by the	*
 *					FIR filter			*
 *----------------------------------------------------------------------*
 *	Returns:	Returns TRUE if job completed sucessfully	*
 *			  (not used in this implementation)		*
 *----------------------------------------------------------------------*
 *	Lookback:	This filter requires a lookback of 36		*
 *----------------------------------------------------------------------*
 *	This routine processes the current record, which has "points"	*
 *	seismic samples, with an FIR filter.  The array "data" 		*
 *	contains these samples plus the last "lookback" values from	*
 *	the previous record (the values from the previous record,	*
 *	that are needed for the FIR filter, are stored in the 0th	*
 *	through lookback-1 positions of the array).  Hence, upon	*
 *	input, the first sample of the current record is stored at	*
 *	index "lookback" of the array.  				*
 *									*
 *	To conserve RAM, we use the same array (the array "data")	*
 *	for both input to the filter and output of it:  Each time	*
 *	we process a new sample, a slot is free above the lookback	*
 *	area.  We fill this slot with the filtered data; hence,		*
 *	when the record has been completely processed, the first	*
 *	"points" slots of the array contain the filtered seismic	*
 *	samples of the record.						*
 *----------------------------------------------------------------------*/

BOOL FfirBB20(point,lookback,data)
WORD point,lookback;
LONG *data;
{

#define cSH_20 >> 5
#define filta(x) (- *(x-1) - *(x-2) - *(x-3) - *(x-4) - *(x-5) \
		- *(x-6) - *(x-7) - *(x-8)) << 2
#define filtb(x) (- *(x-9) - *(x-10) - *(x-11) - *(x-12) + *(x-19) \
		+ *(x-20) + *(x-21)) << 1
#define filtc(x) (- *(x-13) - *(x-14) + *(x-22) + *(x-23) + *(x-24) \
		+ *(x-25) + *(x-26))
#define filtd(x) (- *(x-15) + *(x-27) + *(x-28) + *(x-29)) >> 1
#define filte(x) (- *(x-16) + *(x-18) + *(x-30) + *(x-31) + *(x-32)) >> 2
#define filtf(x) ((*(x-33) + *(x-34))>>3) + (*(x-35)>>4) \
	+ ((*(x-36) - *(x-17))>>5) + (*x<<5)

	LONG *ldp,result;
	WORD i;

	ldp= &data[lookback];  /* the address of the first sample of the
					current record */

	for (i=0; i<point; i++) {  /*process all of the data of the 
					current record */

		result=filta(ldp); /* implemented on different lines because
					some compilers don't like long macros */
		result+=filtb(ldp);
		result+=filtc(ldp);
		result+=filtd(ldp);
		result+=filte(ldp);
		result+=filtf(ldp);

		result=result cSH_20;
		ldp++;  		/* address of next sample */

		data[i]=result; /* store the result in the slot just above
					the lookback of the next sample */

	}

	return(TRUE);

}
