#include <detect.h>

/*----------------------------------------------------------------------*
 *	Function: BOOL FfirSP10(points,lookback,data) - 10SPS SP fir 	*
 *----------------------------------------------------------------------*
 *	Arguments:	WORD points - number of data points		*
 *			WORD lookback - amount of lookback present	*
 *			LONG *data - data array in question 		*
 *----------------------------------------------------------------------*
 *	Returns:	Returns TRUE if job completed sucessfully	*
 *----------------------------------------------------------------------*
 *	Lookback:	This filter requires at least a 3 lookback	*
 *----------------------------------------------------------------------*
 *	This routine processes the current record, which has "points"	*
 *	seismic samples, with an FIR filter.  The array "data" 		*
 *	contains these samples plus the last "lookback" values from	*
 *	the previous record (the values from the previous record,	*
 *	that are needed for the FIR filter, are stored in the 0th	*
 *	through lookback-1 positions of the array).  Hence, upon	*
 *	input, the first sample of the current record is stored at	*
 *	index "lookback" of the array.  				*
 *									*
 *	To conserve RAM, we use the same array (the array "data")	*
 *	for both input to the filter and output of it:  Each time	*
 *	we process a new sample, a slot is free above the lookback	*
 *	area.  We fill this slot with the filtered data; hence,		*
 *	when the record has been completely processed, the first	*
 *	"points" slots of the array contain the filtered seismic	*
 *	samples of the record.						*
 *----------------------------------------------------------------------*/

BOOL FfirSP10(point,lookback,data)
WORD point,lookback;
LONG *data;
{

#define aSH_10	>>3     /* shift for 10 samples/sec filter */
#define filta(x) (*x<<2) - (*(x-2)) - (*(x-3))

	LONG *ldp,result;
	WORD i;

	ldp= &data[lookback];

	for (i=0; i<point; i++) {

		result=filta(ldp);

		result=result aSH_10;
		ldp++;

		data[i]=result;

	}

	return(TRUE);

}
