#include <detect.h>

/*----------------------------------------------------------------------*
 *	Function: BOOL FfirSP20(points,lookback,data) - 20SPS SP fir 	*
 *----------------------------------------------------------------------*
 *	Arguments:	WORD points - number of data points in the	*
 *					the current record		*
 *			WORD lookback - amount of lookback required	*
 *					 for this filter		*
 *			LONG *data - array that contains the seismic	*
 *					samples to be processed by the	*
 *					FIR filter			*
 *----------------------------------------------------------------------*
 *	Returns:	Returns TRUE if job completed sucessfully	*
 *			  (not used in this implementation)		*
 *----------------------------------------------------------------------*
 *	Lookback:	This filter requires a lookback of 7		*
 *----------------------------------------------------------------------*
 *	This routine processes the current record, which has "points"	*
 *	seismic samples, with an FIR filter.  The array "data" 		*
 *	contains these samples plus the last "lookback" values from	*
 *	the previous record (the values from the previous record,	*
 *	that are needed for the FIR filter, are stored in the 0th	*
 *	through lookback-1 positions of the array).  Hence, upon	*
 *	input, the first sample of the current record is stored at	*
 *	index "lookback" of the array.  				*
 *									*
 *	To conserve RAM, we use the same array (the array "data")	*
 *	for both input to the filter and output of it:  Each time	*
 *	we process a new sample, a slot is free above the lookback	*
 *	area.  We fill this slot with the filtered data; hence,		*
 *	when the record has been completely processed, the first	*
 *	"points" slots of the array contain the filtered seismic	*
 *	samples of the record.						*
 *----------------------------------------------------------------------*/

BOOL FfirSP20(point,lookback,data)
WORD point,lookback;
LONG *data;
{

#define aSH_20 >>3     /* shift for 20 samples/sec filter */
#define filta(x) (*x<<1) + (*(x-1)<<2) + (*(x-2)<<1) \
		 - (*(x-5)) -(*(x-6)<<1) -(*(x-7))

	LONG *ldp,result;
	WORD i;

	ldp= &data[lookback];  /* the address of the first sample of the
					current record */

	for (i=0; i<point; i++) {  /*process all of the data of the 
					current record */

		result=filta(ldp); /* implemented on different lines because
					some compilers don't like long macros */

		result=result aSH_20;
		ldp++;  		/* address of next sample */

		data[i]=result; /* store the result in the slot just above
					the lookback of the next sample */

	}

	return(TRUE);

}
