#include <detect.h>

/*----------------------------------------------------------------------*
 *	Function: BOOL FfirSP40(points,lookback,data) - 40SPS SP fir	*
 *----------------------------------------------------------------------*
 *	Arguments:	WORD points - number of data points		*
 *			WORD lookback - amount of lookback present	*
 *			LONG *data - data array in question 		*
 *----------------------------------------------------------------------*
 *	Returns:	Returns TRUE if job completed sucessfully	*
 *----------------------------------------------------------------------*
 *	Lookback:	This filter requires at least a 16 lookback	*
 *----------------------------------------------------------------------*
 *	This routine processes the current record, which has "points"	*
 *	seismic samples, with an FIR filter.  The array "data" 		*
 *	contains these samples plus the last "lookback" values from	*
 *	the previous record (the values from the previous record,	*
 *	that are needed for the FIR filter, are stored in the 0th	*
 *	through lookback-1 positions of the array).  Hence, upon	*
 *	input, the first sample of the current record is stored at	*
 *	index "lookback" of the array.  				*
 *									*
 *	To conserve RAM, we use the same array (the array "data")	*
 *	for both input to the filter and output of it:  Each time	*
 *	we process a new sample, a slot is free above the lookback	*
 *	area.  We fill this slot with the filtered data; hence,		*
 *	when the record has been completely processed, the first	*
 *	"points" slots of the array contain the filtered seismic	*
 *	samples of the record.						*
 *----------------------------------------------------------------------*/

BOOL FfirSP40(point,lookback,data)
WORD point,lookback;
LONG *data;
{

#define bSH_40 >>4
#define filta(x) (- *(x-10) - *(x-12) - *(x-14) - *(x-16))
#define filtb(x) (*x + *(x-2) + *(x-4) + *(x-6) - *(x-11) - *(x-12) \
	- *(x-14) - *(x-15)) << 1
#define filtc(x) (*(x-1) + *(x-2) + *(x-4) + *(x-5) - *(x-13)) << 2
#define filtd(x) *(x-3) << 3

	LONG *ldp,result;
	WORD i;

	ldp= &data[lookback];

	for (i=0; i<point; i++) {

		result=filta(ldp);
		result+=filtb(ldp);
		result+=filtc(ldp);
		result+=filtd(ldp);

		result=result bSH_40;
		ldp++;

		data[i]=result;

	}

	return(TRUE);

}
