/*----------------------------------------------------------------------*
 *  C h i n a   D i g i t a l   S e i s m o g r a p h   N e t w o r k	*
 *									*
 *	CSCONT.H - Main structure definitions				*
 *									*
 *	Contains information used to analyze seismic events.  There	*
 *	is one of these buffers for each discrete component of seismic	*
 *	data that is being event detected.				*
 *----------------------------------------------------------------------*/

#undef min

/*---------------------------Continuity Structure-----------------------*/

struct con_sto {
/*-----------------Variables used in e_detect------------------------------*/

	TEXT *ch_name;		/* Name of detector stored in e_detect 	*/
	WORD sam_ch;		/* Total number samples per channel	 *
				 * of the record			*/
	BYTE cur_rec;		/* Index of the current record */

/*-------------Variable Used in event subroutine-------------------------*/

	WORD buf_flg[E_B];	/* flags: 1 if >= th2; 2 if >= th1 */
	WORD buf_sc[E_B];	/* summed delta sample counts */
	LONG buf_amp[E_B];	/* P-T amplitudes values */
	WORD buf_tim[E_B];	/* time coordinate of P-T values */
	BYTE buf_rec[E_B];	/*  record number array */

	WORD abuf_sc[4];	/* last 4 delta sample counts */
	LONG abuf_amp[4];	/* last 4 P-T amplitudes */
	WORD abuf_tim[4];	/* time cord. of last 4 P-T values */
	BYTE abuf_rec[4];	/* companion to buf_rec */

	LONG last_amp;          /* last P-T value       */

	BOOL epf;		/* event possible flag */
	BOOL evon;		/* event detected flag */
	BOOL icheck;		/* flag, ensures period estimate */

	WORD filhi;		/* see OF Report 83-785 */
	WORD fillo;		/* see OF Report 83-785 */
	WORD iwin;		/* window length (samples) */
	WORD n_hits;		/* # P-T >= th2 for detection  */

	WORD fst_flg;		/* index to first flagged P-T value  */
	WORD indx;		/* saves fst_flg */
	WORD lst_flg;		/* index to last flagged P-T value */
	WORD lst_flg2;		/* lst_flg corrected for overflow */
	WORD lst_pt;		/* index to last P-T value processed */
	WORD lst_pt2;		/* lst_pt corrected for overflow */
	WORD index2;		/* counter,checked for overflow */

	BOOL iset;		/* flag, = 1 when P-T >= th1 */

	WORD jj;		/* index for abrev buffers abuf */
	WORD sumdsc;		/* sum delta sample count */
	WORD sumflg;		/* number of P-T values > th2 */
	WORD last_sc;		/* loaded to abuf_sc[] */
	WORD last_tim;		/* loaded to abuf_tim[] */
	BYTE last_rec;		/*  record of last_tim */

	LONG th1;		/* largest detection threshold */
	LONG th2;		/* smallest detection threshold */

/*---------------------Variables used in p_one----------------------------*/

	LONG last_y;		/* amplitude coordinate of previous sample */
	WORD last_x;		/* time coordinate of previous sample */
	BYTE rec_last_x;	/*  Record of last time coordinate */
	WORD sum_s_c;		/* samples from last P or T to current sample */
	WORD s_sum_sc;		/* samples between last two P-T values */
	WORD index;		/* counter, calls ptwo */
	LONG max_y;		/* amplitude coordinate of P or T value */
	WORD tim_of_max;	/* time coordinate of P or T value */
	BYTE rec_of_max;	/*  record of tim_of_max */
	BOOL prev_slope;	/* sign of last difference  */
	LONG maxamp;		/* abs max of 20 consec. P-T values < thx */
	LONG thx;		/* upper bound for noise est. */
	LONG s_amp;		/* signed amplitude of P-T value */

/*----------------------Variables used in p_two---------------------------*/

	LONG tsstak[16];	/* contains set of maxamp values */
	LONG twosd;		/* statistical dispersion of P-T values */
	WORD kk;		/* index for tsstak[] */
	WORD val_avg;		/* the number of values in tsstak[] */

/*----------------------Variables used in onsetq--------------------------*/

	LONG th3;		/* threshold for estimating onset */
	LONG def_tc;		/* time correction for onset (default) */
	BOOL ponset;		/* Print onsets? */
	VOID (*onsproc)();	/* Call subroutine upon onset */

/*----------------------Variables used in count_dn-------------------------*/

	UWORD wait_blk;		/* controls re-activation of detector and *
				 *  recording time in event coda 	*/
	WORD itc;		/* counter for interval of the event */
	WORD nn;		/* counter for itc */

/*----------------------Variables used in xth-----------------------------*/

	UWORD xth1; 		/* coded threshold factor -- xth*/
	UWORD xth2;		/*           "			*/
	UWORD xth3;		/*           "			*/
	UWORD xthx;		/*           "			*/
				/* th1>th2>thx>th3 or th1>thx>th2>th3	*/

/*----------------------Variables to set up the detector----------------------*/

	WORD kont_per;         /* counts per period */
	LONG ms_sam;		/* milliseconds per sample */
	WORD haf_per;		/* samples per one-half period */
	WORD sam_sec;		/* sample rate of the digital data */
	LONG s_r_x_1000;          /* Sample rate * 1000 */

};

/*------------------------Time Structure-------------------------------*/

struct	r_time	{
	LONG day_yr;
	LONG prv_yr;
	LONG yr;
	LONG day;
	LONG hr;
	LONG min;
	LONG sec;
	LONG msec;
};

/*-------------------------Call interface----------------------------------*/

struct detect_info {
	LONG	samrte;		/* Sample rate * 1000 this detector */
	WORD 	drectyp;	/* Record type designator */
	WORD 	datapts;	/* Number of data points last decode */
	struct r_time startt;	/* Starting time of first data point */
	LONG	*indatar;	/* ptr to data array */
	WORD	lbksize;	/* Space reserved for lookback */
	LONG	*lbkarr;	/* ptr to lookback storage array */
	struct con_sto *incontd; /* This detector's continuity structure */
	TEXT	*detname;	/* Name of this event detector */
	BOOL	(*filterc)();	/* Pointer to the filter function (or NULL) */
};

/*----------------------Auxillarly onset information-----------------------*/

struct s_onset {
	BOOL	new_onset;	/* Flag TRUE if new onset here */
	TEXT	o_polar;	/* A 'd' or 'c' for polarity of first break */
	TEXT	o_dalgo;	/* Detection algorithm 'A' or 'B' */
	UBYTE	o_snr[5];	/* SNR onset quality estimate values */
	UWORD	o_year;		/* Year of onset */
	UWORD	o_days;
	UWORD	o_hours;
	UWORD	o_mins;
	UWORD	o_secs;
	UWORD	o_msecs;
	ULONG	o_amps;		/* Amplitude of signal */
	UWORD	o_pl;		/* Integer part of period */
	ULONG	o_pr;		/* Next two decimals of period */
	ULONG	o_large;	/* twosd, background noise estimate */
};

/*----------------Macros for convenience--------------------------*/

#define	filhi_set(detector,value) (detector)->incontd->filhi = (value)
#define	fillo_set(detector,value) (detector)->incontd->fillo = (value)
#define iwin_set(detector,value) (detector)->incontd->iwin = (value)
#define	nhits_set(detector,value) (detector)->incontd->n_hits = (value)

#define	xth1_set(detector,value) (detector)->incontd->xth1 = (value)
#define	xth2_set(detector,value) (detector)->incontd->xth2 = (value)
#define	xth3_set(detector,value) (detector)->incontd->xth3 = (value)
#define	xthx_set(detector,value) (detector)->incontd->xthx = (value)

#define	deftc_set(detector,value) (detector)->incontd->def_tc = (value)
#define	wait_set(detector,value) (detector)->incontd->wait_blk = (value)
#define	avg_set(detector,value) (detector)->incontd->val_avg = (value)

#define setallparams(cs,fhi,flo,iw,nht,x1,x2,x3,xx,tc,wa,av) \
 { filhi_set(cs,fhi); fillo_set(cs,flo); iwin_set(cs,iw); nhits_set(cs,nht); \
   xth1_set(cs,x1); xth2_set(cs,x2); xth3_set(cs,x3); xthx_set(cs,xx); \
   deftc_set(cs,tc); wait_set(cs,wa); avg_set(cs,av); }

