#include <detect.h>

/*----------------------------------------------------------------------*
 *	Function: BOOL E_cdsnload(detector,indata,offset) - load cdsn	*
 *----------------------------------------------------------------------*
 *	Arguments:	struct detect_info *detector - calling info	*
 *			UBYTE *indata - where the data is		*
 *			WORD offset - what multiplexed channel		*
 *----------------------------------------------------------------------*
 *	Returns:	TRUE - Data valid and converted			*
 *			FALSE - the recordtype did not match		*
 *----------------------------------------------------------------------*
 *	Fatal Errors:	This routine does not have fatal exits		*
 *----------------------------------------------------------------------*
 *	Examine a CDSN record in its internal format.  Determine the	*
 *	record type from the sample rate and compare the record type	*
 *	with the selection criteria in the structure "detector".  If 	*
 *	the record types are the same, decode the rest of the data and  *
 *	return TRUE. Otherwise, do no more, and return FALSE 		*
 *	immediately.							*
 *----------------------------------------------------------------------*/

#define SAM_RC 1014	     /* Size of the data in words */
#define CDSN_EPOCH 83	     /* No data before 1983 */
#define CDSN_CENT 1900	     /* This is the 20th century */

#define IUBYTE(x)	((x)&0xFF)			/* Low order byte */
#define	HINIB(x)	((IUBYTE(indata[x])>>4)&0xF)	/* High order nibble */
#define LONIB(x)	(IUBYTE(indata[x])&0xF)		/* Low order nibble */

BOOL E_cdsnload(detector, indata, offset)
struct detect_info *detector;
UBYTE * indata;
WORD offset;
{

	WORD i, rect, numcomp, leap;
	LONG samrat, sampr, nmsec, fmsec, fsec, fmin, fhour;
	BOOL nak;

	WORD lp, j, gr, ct;
	LONG l_data, *oarray;
	UBYTE * bytarr;

	samrat = HINIB(16) * 10000L +
	    LONIB(16) * 1000L + HINIB(17) * 100L;

	i = samrat / 10;
	switch (i) {
	    case 4000: 		/* 40 samples per second */
	        rect = 1;
	        break;
	    case 2000: 		/* 20 samples per second */
	        rect = 2;
	        break;
	    case 100: 		/*  1 sample per second */
	        rect = 3;
	        break;
	    case 10: 		/* 0.1 sample per second */
	        rect = 4;
	        break;
	    default:		/* Unknown, therefore illegal sample rate */ 
	        rect = 0;
	        break;
	}

	if (detector->drectyp != rect)
	    return(FALSE);		       /* Not this one */

	detector->samrte = samrat;	       /* Detector needs this */

	numcomp = LONIB(8);	/* Number of components in this record */

/*-------------------Compute a complete date-----------------------------*/

	i = LONIB(1) * 10 + HINIB(2);
	if (i < CDSN_EPOCH)
	    i += 100;
	i += CDSN_CENT;

	if (i != detector->startt.yr) {
	    leap = 0;
	    detector->startt.yr = i;
	    if (!(i % 4))
	        leap = 1;
	    if (!(i % 100))
	        leap = 0;
	    if (!(i % 400))
	        leap = 1;
	    detector->startt.day_yr = 365 + leap;
	    i = i - 1;
	    if (!(i % 4))
	        leap = 1;
	    if (!(i % 100))
	        leap = 0;
	    if (!(i % 400))
	        leap = 1;
	    detector->startt.prv_yr = 365 + leap;
	}

	detector->startt.day = LONIB(2) * 100 + HINIB(3) * 10 + LONIB(3);
	detector->startt.hr = HINIB(4) * 10 + LONIB(4);
	detector->startt.min = HINIB(5) * 10 + LONIB(5);
	detector->startt.sec = HINIB(6) * 10 + LONIB(6);
	detector->startt.msec = HINIB(7) * 100 + LONIB(7) * 10;

	bytarr = &indata[20];
	oarray = &detector->indatar[detector->lbksize];

	ct = 0;

	if (offset > numcomp)
	    offset = numcomp;		/* Offset specified is illegal */
	if (offset < 0)
	    offset = 0;			/* Offset illegal */

/*------------------------Convert data from gain-ranged--------------------*/

	for (lp = offset * 2; lp < (SAM_RC * 2); lp +=
	        numcomp * 2) {

	    j = (bytarr[lp] & 0x3F) << 8;	/* Strip off exponent */
	    j |= (bytarr[lp + 1] & 0xFF);

	    j -= 8191;				/* Form signed value */

	    l_data = (LONG) j;

	    gr = (bytarr[lp] & 0xC0) >> 6;	/* Get the exponent */

	    switch (gr) {	/* Apply exponent to form 32 bit integer */
	        case 1: 
	    	    l_data <<= 2;
	    	    break;

	        case 2: 
	    	    l_data <<= 4;
	    	    break;

	        case 3: 
	    	    l_data <<= 7;
	    	    break;

	    }

	    *oarray++ = l_data;		/* Load data into output array */
	    ct++;			/* increment count */
	}
	detector->datapts = ct;		/* Save count */

	return(TRUE);			/* Sucess */

}
