#include <detect.h>

/*----------------------------------------------------------------------*
 *	Function: BOOL E_sroload(detector,indata,offset) - load cdsn	*
 *----------------------------------------------------------------------*
 *	Arguments:	struct detect_info *detector - calling info	*
 *			UBYTE *indata - where the data is		*
 *			WORD offset - what multiplexed channel		*
 *----------------------------------------------------------------------*
 *	Returns:	TRUE - Data valid and converted			*
 *			FALSE - the recordtype did not match		*
 *----------------------------------------------------------------------*
 *	Fatal Errors:	This routine does not have fatal exits		*
 *----------------------------------------------------------------------*
 *	Examine a CDSN record in its internal format.  Determine the	*
 *	record type from the sample rate and compare the record type	*
 *	with the selection criteria in the structure "detector".  If 	*
 *	the record types are the same, decode the rest of the data and  *
 *	return TRUE. Otherwise, do no more, and return FALSE 		*
 *	immediately.							*
 *----------------------------------------------------------------------*/

#define	HINIB(x)	((IUBYTE(indata[x])>>4)&0xF)	/* High order nibble */
#define LONIB(x)	(IUBYTE(indata[x])&0xF)		/* Low order nibble */

BOOL E_sroload(detector, indata, offset)
struct detect_info *detector;
UBYTE * indata;
LONG offset;
{

	LONG i, rect, numcomp, leap;
	LONG samrat, numsam, nmsec, fmsec, fsec, fmin, fhour;
	BOOL nak;

	WORD lp, j, gr, ct;
	LONG l_data, *oarray;
	UBYTE * bytarr;

	numcomp = HINIB(8) * 10 + LONIB(8);	/* Number of components */
	samrat = HINIB(1) * 10000L +		/* Extract sample rate */
	    LONIB(1) * 1000L;

	/* Determine the number of data samples in record */

	numsam = 990;
	if (samrat==20000) {
		numsam = 960;
		if (numcomp==1)
			numsam = 980;
	}

	/* Get the channel id number from the sample rate */

	i = samrat / 10;

	switch (i) {
	    case 2000: 		/* 20 samples per second SH */
	        rect = 1;
	        break;
	    case 100: 		/* 1 samples per second LH */
	        rect = 2;
	        break;
	    default:		/* Unknown, therefore illegal sample rate */
	        rect = 0;
	        break;
	}

	if (detector->drectyp != rect)
	    return(FALSE);		      /* Not this one */

	detector->samrte = samrat;	       /* Detector needs this */

/*-------------------Compute a complete date-----------------------------*/

	i = HINIB(2);
	i += 1980;		/* Correct from 1980 to 1990 */

	if (i != detector->startt.yr) {
	    leap = 0;
	    detector->startt.yr = i;
	    if (!(i % 4))
	        leap = 1;
	    if (!(i % 100))
	        leap = 0;
	    if (!(i % 400))
	        leap = 1;
	    detector->startt.day_yr = 365 + leap;
	    i = i - 1;
	    if (!(i % 4))
	        leap = 1;
	    if (!(i % 100))
	        leap = 0;
	    if (!(i % 400))
	        leap = 1;
	    detector->startt.prv_yr = 365 + leap;
	}

	detector->startt.day = LONIB(2) * 100 + HINIB(3) * 10 + LONIB(3);
	detector->startt.hr = HINIB(4) * 10 + LONIB(4);
	detector->startt.min = HINIB(5) * 10 + LONIB(5);
	detector->startt.sec = HINIB(6) * 10 + LONIB(6);
	detector->startt.msec = HINIB(7) * 100 + LONIB(7) * 10;

	bytarr = &indata[20];
	oarray = &detector->indatar[detector->lbksize];

	ct = 0;

	if (offset > numcomp)
	    offset = numcomp;		/* Offset specified is illegal */
	if (offset < 0)
	    offset = 0;			/* Offset illegal */

/*------------------------Convert data from gain-ranged--------------------*/

	for (lp = offset * 2; lp < (numsam * 2); lp +=
	        numcomp * 2) {

	    /* Decode the gain ranging */

	    j = (bytarr[lp] & 0x0F) << 8;	/* Strip off exponent */
	    j |= (bytarr[lp + 1] & 0xFF);

	    if (j > 0x800) j -= 4096;
	    gr = (bytarr[lp] & 0xF0) >> 4;

	    l_data = (LONG) j;
	    l_data <<= (10-gr);

	    *oarray++ = l_data;		/* Load data into output array */
	    ct++;			/* increment count */
	}

	detector->datapts = ct;		/* Save count */

	return(TRUE);			/* Sucess */

}
